%define ALD_Ver		1.2.3

Name:         openssl

License:      Freely distributable (OpenSSL/SSLeay)
Summary:      Secure Sockets Layer and cryptography libraries and tools
Group:        Development/Libraries
Version:      0.9.8zg
Release:      0
URL: http://www.openssl.org/
#Source0: ftp://ftp.openssl.org/source/%{name}-%{version}.tar.gz


Source: %{name}-%{version}.tar.xz
Source1: CROSS.sh

%define DestDir		/crypt
# -----------------------------------------------------
%if %{_target_cpu} == %{_build_cpu}
%define _CROSS 0
%define BUILD_LOCAL_CROSS 0
%else
# rpm --target=x86_64/armv6l/win32 -ba xxxxx.spec
%define _CROSS 1
%define BUILD_LOCAL_CROSS 1
%endif

%define TO_ARCH %{_target_cpu}

%if %{_target_cpu} == "armv6l"
%define ARM_ARCH armv6zk
%define ARM_FPU  -mfpu=vfp -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%if %{_target_cpu} == "armv7l"
%define ARM_ARCH armv7-a
%define ARM_FPU  -mfpu=vfpv3-d16 -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%define BUILD_TAR 1
#------------------ CROSS DIR --------------------
%if %{_CROSS}

%if %{_os} == "mingw"
ERROR: target=%{_os}
%define target %{_arch}-%{_vendor}-%{_os}%{__isa_bits}
%else  #win32 || win64
%define target %{_arch}-%{_os}%{_gnu}
%endif
%define target_  %{target}-

%define crossdir	/cross
%if %{_arch} == "arm"
%define csysdir		%{crossdir}/%{ARM_ARCH}
%else
%define csysdir		%{crossdir}/%{_target_cpu}
%endif
%define csysbindir	%{csysdir}/bin

%define __os_install_post %{nil}

%define cross_bindir	%{csysdir}%{_bindir}
%define cross_libdir	%{csysdir}%{_libdir}
%define cross_includedir %{csysdir}%{_includedir}
%define cross_datadir	%{csysdir}%{_datadir}
%define cross_infodir	%{csysdir}%{_infodir}
%define cross_mandir	%{csysdir}%{_mandir}
%define cross_defaultdocdir %{csysdir}%{_defaultdocdir}
%define cross_docdir	%{cross_defaultdocdir}/%{name}

%define CrossDestDir	/Cross

%define _srcrpmdir	%{_topdir}/SRPMS/tmp

%else  #_CROSS

%define csysbindir	%{_prefix}/bin
%define CrossDestDir	%{nil}
%define _srcrpmdir	%{_topdir}/SRPMS/%{ALD_Ver}

%define target %{_build}
%define target_  %{nil}

%endif # !_CROSS

%define docdir %{_defaultdocdir}/%{name}
%define openssldir %{_libdir}/ssl

# ------------------- description ------------------------------------
%description
The OpenSSL Project is a collaborative effort to develop a robust,
commercial-grade, fully featured, and Open Source toolkit implementing the
Secure Sockets Layer (SSL v2/v3) and Transport Layer Security (TLS v1)
protocols as well as a full-strength general purpose cryptography library.
The project is managed by a worldwide community of volunteers that use the
Internet to communicate, plan, and develop the OpenSSL tookit and its related
documentation. 

OpenSSL is based on the excellent SSLeay library developed from Eric A.
Young and Tim J. Hudson.  The OpenSSL toolkit is licensed under an
Apache-style licence, which basically means that you are free to get and
use it for commercial and non-commercial purposes. 

This package contains the base OpenSSL cryptography and SSL/TLS 
libraries and tools.
# --------------------------------------
%package devel
Summary:      Libraries for the C compiler
Group:        Development/Libraries
%description devel
These libraries are needed to compile C code.
# --------------------------------------
%package devel-man
Summary:      man for Libraries
Group:        Development/Libraries
%description devel-man
man for Libraries
# --------------------------------------
%package staticlib
Summary:      %{name} static Libraries
Group:        Development/Libraries
%description staticlib
%{name} static Libraries
#-------------------------------
%package perl
Summary:      perl script
Group:        Applications/Publishing
%description perl
perl script

#-------------------------------
%package %{TO_ARCH}
Summary:      %{name} Cross-%{TO_ARCH} Libraries
Group:        Development/Libraries
%description %{TO_ARCH}
%{name} Cross-%{TO_ARCH} Libraries
# --------------------------------------
%package %{TO_ARCH}-staticlib
Summary:      %{name} Cross-%{TO_ARCH} static Libraries
Group:        Development/Libraries
%description %{TO_ARCH}-staticlib
%{name} Cross-%{TO_ARCH} static Libraries

# ------------------- setup ------------------------------------------
%Prep
%setup -q -n %{name}-%{version}

%if %{_arch} == "i386"
OPTFLAGS="-O2 -march=i486 -mpreferred-stack-boundary=2 -falign-functions=1 -falign-jumps=1 -falign-loops=1 -fomit-frame-pointer"
%endif
%if %{_arch} == "x86_64"
OPTFLAGS="-O2 -fPIC -fno-asynchronous-unwind-tables"
%endif
%if %{_arch} == "arm"
OPTFLAGS="-O2 -march=%{ARM_ARCH} %{ARM_FPU} -munaligned-access"
%endif

sed -i s:"-DTERMIO -O3 ":"-DTERMIO $OPTFLAGS ": Configure

# ------------------- Build ------------------------------------------
%Build
%if %{_CROSS}
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
export PATH GCC_NoHostPath
%else
export PATH=$PATH:/opt/perl/bin
%endif

perl util/perlpath.pl /usr/bin/perl

./Configure --prefix=%{_prefix} \
	--libdir=/%{_lib} \
	--openssldir=%{openssldir} \
%if %{_CROSS}
	--cross-compile-prefix="%{target_}" \
%endif
	-DSSL_ALLOW_ADH \
%ifarch %{ix86}
	linux-elf \
%endif
%ifarch x86_64
	linux-x86_64 \
%endif
%ifarch %{arm}
	linux-armv4 \
%endif
	shared \
	threads zlib \
	&>Fconfigure_log.txt

make &>FMake_log.txt

# ------------------- Install ----------------------------------------
%Install
%if %{_CROSS}
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
export PATH GCC_NoHostPath
%else
export PATH=$PATH:/opt/perl/bin
%endif

rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT
mkdir -p %{_srcrpmdir}
mkdir -p $RPM_BUILD_ROOT%{docdir}

make MANDIR=%{_mandir} INSTALL_PREFIX=$RPM_BUILD_ROOT install &>FInstall_log.txt

cp LICENSE README doc/openssl.txt $RPM_BUILD_ROOT%{docdir}/
cp -r doc/HOWTO $RPM_BUILD_ROOT%{docdir}/

mkdir -p $RPM_BUILD_ROOT/etc/ssl
mv $RPM_BUILD_ROOT%{openssldir}/certs $RPM_BUILD_ROOT/etc/ssl/
mv $RPM_BUILD_ROOT%{openssldir}/private $RPM_BUILD_ROOT/etc/ssl/
mv $RPM_BUILD_ROOT%{openssldir}/openssl.cnf $RPM_BUILD_ROOT/etc/ssl/

pushd $RPM_BUILD_ROOT%{openssldir} >/dev/null
ln -s /etc/ssl/certs
ln -s /etc/ssl/private
ln -s /etc/ssl/openssl.cnf
popd >/dev/null

%if %{_CROSS}
TARGET_PRFIX=%{_prefix}
TARGET_CROSS_PRFIX=%{csysdir}
TARGET_NAME=%{target}
MKCROSSDIR=%{BUILD_LOCAL_CROSS}
export TARGET_PRFIX TARGET_CROSS_PRFIX TARGET_NAME MKCROSSDIR
%SOURCE1
%if %{BUILD_LOCAL_CROSS}
rm -r $RPM_BUILD_ROOT%{cross_bindir}
rm -r $RPM_BUILD_ROOT%{cross_mandir}
rm -r $RPM_BUILD_ROOT%{cross_libdir}/engines
rm -r $RPM_BUILD_ROOT%{cross_libdir}/ssl
rm -r $RPM_BUILD_ROOT%{cross_docdir}/HOWTO
rm $RPM_BUILD_ROOT%{cross_docdir}/openssl.txt
rm -r $RPM_BUILD_ROOT%{csysdir}/etc
%endif #BUILD_LOCAL_CROSS
%endif #_CROSS


# ------------------- Clean ------------------------------------------
%Clean 

DE=%{version}-%{release}.%{TO_ARCH}.rpm
DD=%{_rpmdir}/%{TO_ARCH}/%{ALD_Ver}%{DestDir}
DEC=%{version}-%{release}.%{_build_cpu}.rpm
DDC=%{_rpmdir}/%{_build_cpu}/%{ALD_Ver}%{CrossDestDir}/%{TO_ARCH}%{DestDir}

pushd %{_rpmdir}/%{_target_cpu}
  SE=%{version}-%{release}.%{_target_cpu}.rpm
%if (%{_os} == "mingw")
  rm %{name}-$SE
%if %{BUILD_LOCAL_CROSS}
  mkdir -p $DDC
  mv %{name}-%{TO_ARCH}-$SE $DDC/%{name}-%{TO_ARCH}-$DEC
%endif #BUILD_LOCAL_CROSS
%else  #mingw
  mkdir -p $DD
  mv %{name}-$SE $DD/%{name}-$DE
  mv %{name}-devel-$SE $DD/%{name}-devel-$DE
  mv %{name}-devel-man-$SE $DD/%{name}-devel-man-$DE
  mv %{name}-staticlib-$SE $DD/%{name}-staticlib-$DE
  mv %{name}-perl-$SE $DD/%{name}-perl-$DE
%if %{BUILD_LOCAL_CROSS}
  mkdir -p $DDC
  mv %{name}-%{TO_ARCH}-$SE $DDC/%{name}-%{TO_ARCH}-$DEC
  mv %{name}-%{TO_ARCH}-staticlib-$SE $DDC/%{name}-%{TO_ARCH}-staticlib-$DEC
%endif #BUILD_LOCAL_CROSS
%endif #!mingw
popd

# --------------- build *.tgz ------------------------
%if %{BUILD_TAR}
pushd $RPM_BUILD_ROOT
%if (%{_os} == "mingw")
  TMP_TO=$DD
%else
  TMP_TO=$DD/tar
%endif
  mkdir -p $TMP_TO
  tmp=%{name}-%{version}-%{release}.%{TO_ARCH}
  mkdir -p $tmp
  cp -r etc usr $tmp/
%if (%{_os} == "mingw")
  zip -9 -r $tmp.zip $tmp >/dev/null
  mv $tmp.zip $TMP_TO
%else
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
%endif
  rm -r $tmp
%if %{BUILD_LOCAL_CROSS}
  TMP_TO=$DDC/tar
  mkdir -p $TMP_TO
  tmp=%{name}-%{TO_ARCH}-%{version}-%{release}.%{_build_cpu}
  mkdir -p $tmp
  cp -r cross $tmp/
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
  rm -r $tmp
%endif
popd
%endif  #BUILD_TAR

# ------------------- Files ------------------------------------------
%Files
/etc
%{_bindir}/openssl
%{_libdir}/engines
%{_libdir}/*.so.*
%{_mandir}/man1
%{_mandir}/man5
%{_mandir}/man7
%{docdir}
%{openssldir}/certs
%{openssldir}/misc/CA.sh
%{openssldir}/misc/c_*
%{openssldir}/private
%{openssldir}/openssl.cnf


%files devel
%{_includedir}
%{_libdir}/pkgconfig
%{_libdir}/*.so

%files devel-man
%{_mandir}/man3

%Files staticlib
%{_libdir}/*.a

%Files perl
%{_bindir}/c_rehash
%{openssldir}/misc/CA.pl

# ------------------- Cross Files ------------------------------------
%if %{BUILD_LOCAL_CROSS}

%Files %{TO_ARCH}
%{cross_includedir}
%{cross_libdir}/pkgconfig
#{cross_libdir}/engines
%{cross_libdir}/*.so.*
%{cross_libdir}/*.so
%{cross_docdir}

%Files %{TO_ARCH}-staticlib
%{cross_libdir}/*.a

%endif #!BUILD_LOCAL_CROSS
